import { useStore } from 'vuex'
import { createDecorator } from 'vue-class-component'

export function State(value:string) {
    const arr = value.split("/");
    return function (target: any, propertyKey: string) {
        createDecorator(function (options, key) {
            options.computed[key] = ()=>{
                let data:any = useStore().state
                arr.forEach((item)=>{
                  data = data[item]
                })
                return data;
            }
        })(target, propertyKey);
    };
}

export function Action(value:string) {
    return function (target: any, propertyKey: string) {
        createDecorator(function (options, key) {
            options.methods[key] = function(...args:any[]) {
                return useStore().dispatch(value,...args)
            }
        })(target, propertyKey);
    };
}

export function Getter(value:string) {
    return function (target: any, propertyKey: string) {
        createDecorator(function (options, key) {
            options.methods[key] = ()=>{
                return useStore().getters(value)
            }
        })(target, propertyKey);
    };
}

export function Mutation(value:string) {
    return function (target: any, propertyKey: string) {
        createDecorator(function (options, key) {
            options.methods[key] = function(...args:any[]) {
                return useStore().commit(value,...args)
            }
        })(target, propertyKey);
    };
}

export default {
    State,
    Action,
    Getter,
    Mutation
}